function [L,A] = ConvexApproLRDD( Sigma,lambda,rho,N)
% The ADMM algorithm for approximate DD-PCA
% Input: Sigma is the matrix. lambda is the paramter in objective function;
% rho is the parameter in ADMM implementation. N is the iteration time;  
% Output: low rank part L and diagonal dominant part A 

p = size(Sigma,1);
% initialization
A = zeros(p);
E = zeros(p);
Lambda = zeros(p);

for i=1:N
    % M-step
    [U,D,V] = svd(Sigma-E-A-Lambda/rho);
    D_s = sign(D).* max(abs(D)-lambda/rho,0);
    L = U*D_s*V';
    
    % A-step
    temp2 = (Sigma-L-E-Lambda/rho);
    A = ProjSDDplus(temp2);
    
    % E-step
    E = rho/(rho+1)*(Sigma-A-L-Lambda/rho);
    
    % Lambda-step
    Lambda = Lambda + rho*(A+L+E-Sigma);
    
end

end

