function [L,A] = ConvexLRDD( Sigma,rho,N )
% The ADMM algorithm for exact DD-PCA
% Input: Sigma is the matrix. rho is the parameter in ADMM implementation. N is the
% iteration time;  
% Output: low rank part L and diagonal dominant part A 

p = size(Sigma,1);
% initialization
A = zeros(p);
Lambda1 = zeros(p);
Lambda2 = zeros(p);  

for i=1:N
    % M-step
    [U,D,V] = svd(Sigma-A-Lambda1/rho);
    D_s = sign(D).* max(abs(D)-1/rho,0);
    L = U*D_s*V';
    
    % B-step
    temp1 = A + Lambda2/rho;
    B = (temp1+temp1')/2;
    
    % A-step
    temp2 = (Sigma-L+B-Lambda1/rho-Lambda2/rho)/2;
    A = ProjSDD(temp2);
    
    % Lambda-step
    Lambda1 = Lambda1 + rho*(A+L-Sigma);
    Lambda2 = Lambda2 + rho*(A-B);
    
end

end

