% This script can be used to produce part of figure 1 in DDPCA paper. 
% For repetition=1, it will take around 0.5 minute.
clear
clc;
rng('default')
%% Covariance Matrix Estimation
t1=tic;
repetition = 100; 
k = 3;
T = 200;
p = 200;
rho = 0.5;
Klist = 1:8;
Klen = length(Klist);
%% Metrics
Frob_DD = zeros(repetition,Klen);
Frob_poet = zeros(repetition,Klen);
Twonorm_DD = zeros(repetition,Klen);
Twonorm_poet = zeros(repetition,Klen);

Frob_InvDD = zeros(repetition,Klen);
Frob_Invpoet = zeros(repetition,Klen);
Twonorm_InvDD = zeros(repetition,Klen);
Twonorm_Invpoet = zeros(repetition,Klen);

RelFrob_DD = zeros(repetition,Klen);
RelFrob_poet = zeros(repetition,Klen);
RelTwonorm_DD = zeros(repetition,Klen);
RelTwonorm_poet = zeros(repetition,Klen);
%% Main Loop
B = randn(p,k);
for i = 1:repetition
    F = randn(k,T);
    Sigma_mu = rho.^abs(repmat((1:p)',1,p) - repmat(1:p,p,1));
    Sigma_mu = (Sigma_mu + eye(p))/2; % Now Sigma_mu is a diagonally-dominant matrix
    Sigma_pop = Sigma_mu + B*B';
    halfinv_Sigma_pop = Sigma_pop^(-1/2);
    Omega_pop = inv(Sigma_pop);
    Omega_mu = inv(Sigma_mu);
    mu = mvnrnd(zeros(T,p),Sigma_mu);
    Y = (B*F)' + mu; % Y is T by p matrix
    Sigma_sample = cov(Y);
    
    for j = 1:Klen
        K_current = Klist(j);
        % POET
        C_min = Cmin(Y',K_current,'hard');
        if isnan(C_min)
            C_min = 0;
        end
        C_best = Cross_val(Y',K_current,'hard',C_min);
        [SigmaY,~]=POET(Y',K_current,C_best,'hard');
        inv_SigmaY = inv(SigmaY);
        
        Frob_poet(i,j) = norm(SigmaY-Sigma_pop,'fro');
        Twonorm_poet(i,j) = norm(SigmaY-Sigma_pop,2);
        Frob_Invpoet(i,j) = norm(inv_SigmaY-Omega_pop,'fro');
        Twonorm_Invpoet(i,j) = norm(inv_SigmaY-Omega_pop,2);
        RelFrob_poet(i,j) = norm(halfinv_Sigma_pop*SigmaY*halfinv_Sigma_pop - eye(p),'fro')/sqrt(p);
        RelTwonorm_poet(i,j) = norm(halfinv_Sigma_pop*SigmaY*halfinv_Sigma_pop - eye(p),2);
        
        % DD
        Sigma_sample = cov(Y);
        [ T_DD,rem_DD ] = LRDD(Sigma_sample,K_current,1);
        rem_DD=ProjSDDplus(rem_DD);
        SigmaY_DD = rem_DD+T_DD;
        invpop_DD=inv(SigmaY_DD);
        
        Frob_DD(i,j) = norm(SigmaY_DD-Sigma_pop,'fro');
        Twonorm_DD(i,j) = norm(SigmaY_DD-Sigma_pop,2);
        Frob_InvDD(i,j) = norm(invpop_DD-Omega_pop,'fro');
        Twonorm_InvDD(i,j) = norm(invpop_DD-Omega_pop,2);
        RelFrob_DD(i,j) = norm(halfinv_Sigma_pop*SigmaY_DD*halfinv_Sigma_pop - eye(p),'fro')/sqrt(p);
        RelTwonorm_DD(i,j) = norm(halfinv_Sigma_pop*SigmaY_DD*halfinv_Sigma_pop - eye(p),2);
        
    end
    
end
%%
save('pop.mat')
toc(t1)