clear
clc;
rng('default');
%% Covariance Matrix Estimation
t1 = tic;
repetition = 100; % For repetition=100, it will take around 15 seconds
k0 = 3;
T = 200;
p = 100;
rho = 0.5;
klist = 1:6;
taulist = linspace(0.1,0.9,17);
%% Metrics
Frob = zeros(repetition,length(klist));
Spectral = zeros(repetition,length(klist));
Frob_Inv = zeros(repetition,length(klist));
Spectral_Inv = zeros(repetition,length(klist));

Frob_poet = zeros(repetition,length(taulist));
Twonorm_poet = zeros(repetition,length(taulist));
Frob_Invpoet = zeros(repetition,length(taulist));
Twonorm_Invpoet = zeros(repetition,length(taulist));
%% Main Loop
for i = 1:repetition
    B = randn(p,k0);
    F = randn(k0,T);
    Sigma_mu = rho.^abs(repmat((1:p)',1,p) - repmat(1:p,p,1));
    Sigma_mu = (Sigma_mu + eye(p))/2; % Now Sigma_mu is a diagonally-dominant matrix
    Omega_mu = inv(Sigma_mu);
    mu = mvnrnd(zeros(T,p),Sigma_mu);
    Y = (B*F)' + mu; % Y is T by p matrix
    Sigma_sample = cov(Y);
    % DD
    for j=1:length(klist)
        k = klist(j);
        [ T_DD,rem_DD ] = LRDD(Sigma_sample,k,1);
        rem_DD=(rem_DD+rem_DD')/2;
        invrem_DD=inv(rem_DD);
        
        Frob(i,j) = norm(rem_DD-Sigma_mu,'fro');
        Spectral(i,j) = norm(rem_DD-Sigma_mu,2);
        Frob_Inv(i,j) = norm(invrem_DD-Omega_mu,'fro');
        Spectral_Inv(i,j) = norm(invrem_DD-Omega_mu,2);
    end
    % POET
    [V,D] = eigs(Sigma_sample,k0);
    T_poet = V*D*V';
    rem_poet = Sigma_sample - T_poet;
    sd = diag(1./sqrt(diag(rem_poet)));
    rem_poet_corr = sd * rem_poet * sd;
    for j = 1:length(taulist)
        tau = taulist(j);
        temp = rem_poet;
        temp(rem_poet_corr<tau) = 0;
        invrem_poet=inv(temp);
        Frob_poet(i,j) = norm(temp-Sigma_mu,'fro');
        Twonorm_poet(i,j) = norm(temp-Sigma_mu,2);
        Frob_Invpoet(i,j) = norm(invrem_poet-Omega_mu,'fro');
        Twonorm_Invpoet(i,j) = norm(invrem_poet-Omega_mu,2);
    end
    
end
%% 
display(mean(Frob))
display(mean(Spectral))
display(mean(Frob_Inv))
display(mean(Spectral_Inv))
%%
display(min(mean(Frob_poet)))
display(min(mean(Twonorm_poet)))
display(min(mean(Frob_Invpoet)))
display(min(mean(Twonorm_Invpoet)))
toc(t1)