function C = Cross_val( Y,K,thres,Cmin)
% Cross-validation to determine threshold C for POET
%   return threshold C
[p,n]=size(Y);
Y= Y-mean(Y,2)*ones(1,n) ; % Y is de-meaned
n1 = floor(n*(1-1/log(n))); 
H = 10;

% find the residuals

if K>0
    [V,~]=eig(Y'*Y);
    index= sort((n-K+1:n),'descend');
    F=V(:,index)*sqrt(n);
    LamPCA=Y*F/n; 
    uhat=Y-LamPCA*F';   % p x n
    rate=1/sqrt(p)+sqrt((log(p))/n);

else
    uhat=Y; % Sigma_y itself is sparse
    rate=sqrt((log(p))/n);
end;

Cmax = 1/rate;
C_try = 20;
C_list = linspace(Cmin + 0.01,Cmax - 0.01,C_try);


result = zeros(H,C_try);
% main loop

for split=1:H
    j1_index = randsample(n,n1);
    j2_index = setdiff(1:n,j1_index);
    
    uhat_1 = uhat(:,j1_index);
    uhat_2 = uhat(:,j2_index);
    
    SuPCA=uhat_1*uhat_1'/n1;
    SuDiag=diag(diag(SuPCA));
    hinv = diag(1./sqrt(diag(SuPCA)));
    R=hinv*SuPCA*hinv;
    
    Sigma_val = uhat_2*uhat_2'/(n-n1);
    
    for i=1:C_try
        C_current = C_list(i);
        lambda=rate*C_current*ones(p,p);
        
        switch thres
            case 'soft'
                M=sign(R).*subplus(abs(R)-lambda);
            case 'hard'
                M=R.*(abs(R)>=lambda);
            case 'scad'
                M1=(abs(R)<2*lambda).*sign(R).*(abs(R)-lambda).*(abs(R)>lambda);
                M2=(abs(R)<3.7*lambda).*(abs(R)>=2*lambda).*((3.7-1)*R-3.7*sign(R).*lambda)/(3.7-2);
                M3=(abs(R)>=3.7*lambda).*R;
                M=M1+M2+M3;
                
        end;
        
        Rthresh=M-diag(diag(M))+eye(p);
        SigmaU=SuDiag^(1/2)*Rthresh*SuDiag^(1/2);
        
        result(split,i) = norm(SigmaU - Sigma_val,'fro')^2;
    
    end
end

[~,C_index] = min(mean(result));

C = C_list(C_index);

end

