% This script can be used to produce figure 6 in DDPCA paper. 
% For N=10 it will run approximately 1.5 hour. 
clear;
clc;
rng('default');
%% setting
t1 = tic;
estimate_covariance = true;
num_copies = 50;
d = 500; % dimension (we reserve notation 'p' for p-values )
s = 25; % sparsity level
N = 100; % number of trials 
K = 2;
r = 2;
N_method = 8;
X = 1/sqrt(r)*randn(d,r);  
L0 = X*X';
rho1 = 0.5;
Sigma = rho1.^abs(repmat((1:d)',1,d) - repmat(1:d,d,1)) + L0;
% normalize Sigma
tD = 1./sqrt(diag(Sigma));
Sigma = diag(tD)*Sigma*diag(tD);
Sigma_inv_vec = sqrt(diag(inv(Sigma)));
%% Methods
inputs = cell(3,2);
if (estimate_covariance==false)
    % Independence-PCA
    [V,D]=eigs(Sigma,K);
    L = V*D*V';
    Rem = Sigma - L;
    D1 = diag(diag(Rem));
    b = V*sqrt(D);
    inputs(1,:) = {b,D1};
    % poet 
    hinv = diag(1./sqrt(diag(Rem)));
    R = hinv*Rem*hinv;
    lambda_R = 0.5; % threshold 
    M = R.*(abs(R)>=lambda_R);
    D2 = D1^(1/2)*M*D1^(1/2);
    inputs(2,:) = {b,D2};
    % DD-PCA
    [T,D3] = LRDD(Sigma,K,15);
    [eigvec,lambda] = eig(T);
    b = eigvec(:,1:K) * sqrt(lambda(1:K,1:K));
    inputs(3,:) = {b,D3};
end
%% main loop
signal_list = 0.2:0.2:3;
if estimate_covariance
    signal_list = signal_list/sqrt(num_copies);
end
Errormatrix = zeros(length(signal_list),N_method);
for j=1:length(signal_list)
    signal_strength = signal_list(j);
    Nulls = zeros(N,N_method); % 'HC','Max','SumOfSquares','Innovated_HC','DD_HC'
    Alts = zeros(N,N_method);
    mu1 = zeros(1,d);
    signal_position = randsample(d,s);
    mu1(signal_position) = signal_strength;
    
    for i=1:N
        if estimate_covariance
            % for null
            data = mvnrnd(zeros(num_copies,d),Sigma);
            mean_data = mean(data,1)';
            Sigma_hat = cov(data)/num_copies;
            Sigma_inv_vec = sqrt(diag(inv(Sigma_hat + 0.1*eye(d))));
            % Independence-IHC
            [V,D]=eigs(Sigma_hat,K);
            L = V*D*V';
            D1 = diag(diag(Sigma_hat - L));
            b = V*sqrt(D);
            inputs(1,:) = {b,D1};
            % POET
            C_min = Cmin(data',K,'hard');
            if isnan(C_min)
                C_min = 0;
            end
            C_best = Cross_val(data',K,'hard',C_min);
            [SigmaY,SigmaU]=POET(data',K,C_best,'hard');
            SigmaY = SigmaY/num_copies;
            SigmaU = SigmaU/num_copies;
            [eigvec,lambda] = eig(SigmaY - SigmaU);
            b = eigvec(:,1:K) * sqrt(lambda(1:K,1:K));
            inputs(2,:) = {b,SigmaU};
            % DD-IHC
            [T,D2] = LRDD(Sigma_hat,K,15);
            [eigvec,lambda] = eig(T);
            b = eigvec(:,1:K) * sqrt(lambda(1:K,1:K));
            inputs(3,:) = {b,D2};
            
            Nulls(i,:) = TestStat(mean_data,Sigma_hat,Sigma_inv_vec,inputs);
            
            % for alternative
            data2 = mvnrnd(repmat(mu1,num_copies,1),Sigma);
            mean_data2 = mean(data2,1)';
            Sigma_hat = cov(data2)/num_copies;
            Sigma_inv_vec = sqrt(diag(inv(Sigma_hat + 0.1*eye(d))));
            % Independence-IHC
            [V,D]=eigs(Sigma_hat,K);
            L = V*D*V';
            D1 = diag(diag(Sigma_hat - L));
            b = V*sqrt(D);
            inputs(1,:) = {b,D1};
            % POET
            C_min = Cmin(data2',K,'hard');
            if isnan(C_min)
                C_min = 0;
            end
            C_best = Cross_val(data2',K,'hard',C_min);
            [SigmaY,SigmaU]=POET(data2',K,C_best,'hard');
            SigmaY = SigmaY/num_copies;
            SigmaU = SigmaU/num_copies;
            [eigvec,lambda] = eig(SigmaY - SigmaU);
            b = eigvec(:,1:K) * sqrt(lambda(1:K,1:K));
            inputs(2,:) = {b,SigmaU};
            
            % DD-IHC
            [T,D2] = LRDD(Sigma_hat,K,15);
            [eigvec,lambda] = eig(T);
            b = eigvec(:,1:K) * sqrt(lambda(1:K,1:K));
            inputs(3,:) = {b,D2};
            
            Alts(i,:) = TestStat(mean_data2,Sigma_hat,Sigma_inv_vec,inputs);
            
        else
            data = mvnrnd(zeros(1,d),Sigma)';
            Nulls(i,:) = TestStat(data,Sigma,Sigma_inv_vec,inputs);
            data2 = mvnrnd(mu1,Sigma)';
            Alts(i,:) = TestStat(data2,Sigma,Sigma_inv_vec,inputs);
        end
    end
    % Metric
    null = sort(Nulls,'ascend');
    sumerror = zeros(N,N_method);
    for i=1:N
        threshold = repmat(null(i,:),N,1);
        sumerror(i,:) = sum(Alts<=threshold)/N + (N-i)/N;
    end
    Errormatrix(j,:) = min(sumerror);
end
toc(t1)
%% plot graph
figure
Colorlist=['r','y','g','k','b','b','b','m'];
Linelist={'-','-','-','-','--',':','-','-'}; 
hold on 
for i=1:N_method
    plot(signal_list,Errormatrix(:,i)','Color',Colorlist(i),'LineStyle',Linelist{i},'LineWidth',3)
end
legend({'HC','ME','\chi^2','IHC','IHC-Inde','IHC-POET','IHC-DD','DD-HC'},'FontSize',18,'Location','northeast')
xlabel('Signal Strength','FontSize',18)
ylabel('Ideal Testing Error','FontSize',18)
title(['(p,s)=' '(' num2str(d) ','  num2str(s) ')' ],'FontSize',20)
set(gca,'FontSize',20)
hold off
%% print
fig = gcf;
fig.PaperUnits = 'inches';
fig.PaperPosition = [0 0 9 6];
fig.PaperSize = [9.25,6.25];
print(['p' num2str(d) 's' num2str(s) '_v2'],'-dpdf','-r250')
%% save workspace
save(['p' num2str(d) 's' num2str(s) '_v2' '.mat'])