% This script can be used to produce Figure 2 in DDPCA paper. Running time
% should be around 2 min.
clear
clc;
rng('default')
%% Data 
t1 = tic;
load('Data/SP100_06to16.mat')
Y = ret_matrix;
days = cumsum(trading_days);
days = [0;days];
n_stock = size(Y,2);
total_month = size(trading_days,1);
init_month = 12;
tau = 0.8;
k = 3;
l = total_month-init_month;
Risk_DD = zeros(1,l);
Risk_poet = zeros(1,l);
%% Main Loop
for i = 1:l
    Start = days(i) + 1;
    End = days(i+init_month);
    test_End = days(i+1+init_month);
    Sigma_sample = cov(Y(Start:End,:));
    
    C_min = Cmin(Y(Start:End,:)',k,'hard');
    if isnan(C_min)
        C_min = 0;
    end
    C_best = Cross_val(Y(Start:End,:)',k,'hard',C_min);
    [Sigma_poet,~]=POET(Y(Start:End,:)',k,C_best,'hard');
    % DD
    [ T_DD,rem_DD ] = LRDD(Sigma_sample,k,1);
    rem_DD=ProjSDDplus(rem_DD);
    Sigma_DD = T_DD + rem_DD;
    % Construct portfolio
    w_poet = Sigma_poet\ones(n_stock,1); 
    w_poet = w_poet/sum(w_poet);
    w_DD = Sigma_DD\ones(n_stock,1); 
    w_DD = w_DD/sum(w_DD);
    % calculate risk
    new_sample = Y((End+1):test_End,:);
    new_cov = new_sample'*new_sample/size(new_sample,1);
    Risk_poet(i) = w_poet'*new_cov*w_poet;
    Risk_DD(i) = w_DD'*new_cov*w_DD;
end
%% Results comparison

Risk_ratio = 100*(Risk_poet - Risk_DD)./Risk_poet;
mean_ratio = mean(Risk_ratio);
median_ratio = median(Risk_ratio);

hold on
histogram(Risk_ratio,15)
line([0 0], get(gca, 'ylim'), 'Color', 'k','LineWidth',4, 'LineStyle', '-')
meanline = line([mean_ratio mean_ratio], get(gca, 'ylim'), 'Color', 'r','LineWidth',4, 'LineStyle', '--');
medianline = line([median_ratio median_ratio], get(gca, 'ylim'), 'Color', 'g','LineWidth',4, 'LineStyle', '-.');
lgd = legend([meanline,medianline],['mean =',num2str(round(mean_ratio,1)),'%'],['median = ',num2str(round(median_ratio,1)),'%']);
lgd.FontSize = 15;
xlabel('percent','FontSize',15)
hold off
%% Print figure
fig = gcf;
fig.PaperUnits = 'inches';
fig.PaperPosition = [0 0 8 6];
fig.PaperSize = [8.25,6.25];
print('Risk_hist','-dpdf','-r250')
%%
toc(t1)