%% Robust Analysis
% This script can be used to produce Figure 10 in DDPCA paper by setting p = 500 and 2000 
clear
clc;
rng('default');
%% setup
t1 = tic;
p = 500;
k0 = p*0.05;
klist = linspace(5,50,10);
error_frob = zeros(length(klist),1);
error_spectral = zeros(length(klist),1);
% data generation
X = 1/sqrt(k0)*randn(p,k0);  
M = X*X';
frob_M = norm(M,'fro');
spectral_M = norm(M,2);
a = 1; % control the size of diagonal entries
A = 1/p*randn(p,p);  
A = A+A';
A = A-diag(diag(A));
A = diag(a*sum(abs(A)))+A;
Sigma = M + A;
%% DDPCA
for i=1:length(klist)
    k = klist(i);
    [L,~] = LRDD(Sigma,k,10);
    error_frob(i) = norm(L-M,'fro')/frob_M;
    error_spectral(i) = norm(L-M,2)/spectral_M;
end
%% plot
figure
hold on 
plot(klist,error_frob,'-or','LineWidth',3)
plot(klist,error_spectral,'--ob','LineWidth',3)
xlabel('k','FontSize',30)
ylabel('Relative error on low rank part','FontSize',20)
title(['p=' num2str(p),',  ', 'K=' num2str(k0)],'FontSize',30)
legend({'Frobenius Norm','Spectral Norm'},'FontSize',18)
set(gca,'FontSize',20)
hold off
fig = gcf;
fig.PaperUnits = 'inches';
fig.PaperPosition = [0 0 9 6];
fig.PaperSize = [9.25,6.25];
print('robust_p500','-dpdf','-r250')
toc(t1)