function [ sorted_index, numselect ] = if_screen(Data, numselect)
%UNTITLED Summary of this function goes here
%   Data: p by n
%   numselect:  1 by l vector
%   return type: p by 1 logical vector

%% Normalize data
[p, n] = size(Data);

gm = mean(Data,2); gsd = std(Data,0,2);
Data = (Data - repmat(gm, 1, n))./repmat(gsd, 1, n);

%% Calculate KS value for each feature in the data set
KS = zeros(p, 1);
parfor j= 1:p
    pi = normcdf(Data(j,:)/sqrt(1 - 1/n));
    pi = sort(pi);
    kk = (0:n)'/n;
    KS(j) = max(max(abs(kk(1:n) - pi')), max(abs(kk(2:(n+1)) - pi')));
end
KS = (KS - mean(KS))/std(KS);

%% if numselect is not provided 

if (nargin<2)
    % simulate KS value
    rep = 100*p;
    KSvalue = zeros(rep,1); 
    parfor i = 1:rep
        x = randn(n,1); 
        z = (x - mean(x))/std(x);
        z = z/sqrt(1 - 1/n);
        pi = normcdf(z);
        pi = sort(pi);
        kk = (0:n)'/n;
        KSvalue(i) = max(max(abs(kk(1:n) - pi)), max(abs(kk(2:(n+1)) - pi)));
    end
    KSvalue = KSvalue*sqrt(n);
    
    KSmean = mean(KSvalue); KSstd = std(KSvalue); 
    
    % Calculate P-value with simulated KS values
    KSadjust = KS*KSstd + KSmean;
    pval = zeros(p,1);
    parfor i = 1:p
        pval(i) = mean(KSvalue > KSadjust(i));
    end
    [psort, ~] = sort(pval, 'ascend');
    
    
    % Calculate HC functional at each data point
    kk = (1:p)'/(1 + p);
    HCsort = sqrt(p)*(kk - psort)./sqrt(kk);
    HCsort  =   HCsort./sqrt(max(sqrt(n)*(kk - psort)./kk, 0) + 1 );
    
    % Decide the threshold
    pvalcut = log(p)/p;
    Ind = find(psort>pvalcut, 1, 'first');
    ratio = HCsort;
    ratio(1:Ind-1) = -Inf; ratio(round(p/2)+1:end)=-Inf;
    L = find(ratio == max(ratio), 1, 'last');
    numselect = L;
end
    
%% sorted indices
[~, sorted_index] = sort(KS,'descend');

end

